part of masamune.universal;

class UniversalAppBar extends StatelessWidget {
  const UniversalAppBar({
    super.key,
    this.leading,
    this.automaticallyImplyLeading = true,
    this.title,
    this.subtitle,
    this.actions,
    this.flexibleSpace,
    this.bottom,
    this.elevation,
    this.shadowColor,
    this.borderColor,
    this.forceElevated = false,
    this.backgroundColor,
    this.foregroundColor,
    this.iconTheme,
    this.actionsIconTheme,
    this.primary = true,
    this.centerTitle,
    this.excludeHeaderSemantics = false,
    this.titleSpacing,
    this.titlePadding = const EdgeInsets.fromLTRB(16, 0, 0, 0),
    this.collapsedHeight,
    this.expandedHeight,
    this.floating = true,
    this.pinned = false,
    this.snap = false,
    this.stretch = false,
    this.stretchTriggerOffset = 100.0,
    this.onStretchTrigger,
    this.shape,
    this.toolbarHeight = kToolbarHeight + 1,
    this.leadingWidth,
    this.toolbarTextStyle,
    this.titleTextStyle,
    this.systemOverlayStyle,
    this.sliverLayoutWhenModernDesign = true,
    this.background,
    this.scrollStyle = UIAppBarScrollStyle.pinned,
    this.type,
  })  : assert(
          floating || !snap,
          'The "snap" argument only makes sense for floating app bars.',
        ),
        assert(stretchTriggerOffset > 0.0),
        assert(
          collapsedHeight == null || collapsedHeight >= toolbarHeight,
          'The "collapsedHeight" argument has to be larger than or equal to [toolbarHeight].',
        );

  final ResponsiveContainerType? type;
  final bool sliverLayoutWhenModernDesign;
  final Widget? background;
  final UIAppBarScrollStyle scrollStyle;
  final EdgeInsetsGeometry titlePadding;

  final Widget? leading;

  final bool automaticallyImplyLeading;

  final Widget? title;

  final Widget? subtitle;

  final List<Widget>? actions;

  final Widget? flexibleSpace;

  final PreferredSizeWidget? bottom;

  final double? elevation;

  final Color? shadowColor;

  final Color? borderColor;

  final bool forceElevated;

  final Color? backgroundColor;

  final Color? foregroundColor;

  final IconThemeData? iconTheme;

  final IconThemeData? actionsIconTheme;

  final bool primary;

  final bool? centerTitle;

  final bool excludeHeaderSemantics;

  final double? titleSpacing;

  final double? collapsedHeight;

  final double? expandedHeight;

  final bool floating;

  final bool pinned;

  final ShapeBorder? shape;

  final bool snap;

  final bool stretch;

  final double stretchTriggerOffset;

  final AsyncCallback? onStretchTrigger;

  final double toolbarHeight;

  final double? leadingWidth;

  final TextStyle? toolbarTextStyle;

  final TextStyle? titleTextStyle;

  final SystemUiOverlayStyle? systemOverlayStyle;

  @override
  Widget build(BuildContext context) {
    if (!sliverLayoutWhenModernDesign) {
      return ResponsiveAppBar(
        key: key,
        leading: leading,
        automaticallyImplyLeading: automaticallyImplyLeading,
        title: subtitle == null
            ? title
            : Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.end,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  if (title != null) title!,
                  DefaultTextStyle(
                    style: Theme.of(context).textTheme.labelSmall?.copyWith(
                              color: Theme.of(context).colorScheme.primary,
                            ) ??
                        TextStyle(
                          color: Theme.of(context).colorScheme.primary,
                        ),
                    child: subtitle!,
                  ),
                ],
              ),
        actions: actions,
        flexibleSpace: flexibleSpace,
        bottom: bottom,
        elevation: elevation ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? 1
                : 0),
        shadowColor: borderColor ?? Theme.of(context).scaffoldBackgroundColor,
        backgroundColor:
            backgroundColor ?? Theme.of(context).appBarTheme.backgroundColor,
        foregroundColor: foregroundColor ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : Theme.of(context).dividerColor),
        iconTheme: iconTheme ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : IconThemeData(color: Theme.of(context).dividerColor)),
        actionsIconTheme: actionsIconTheme ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : IconThemeData(color: Theme.of(context).dividerColor)),
        primary: primary,
        centerTitle: centerTitle,
        excludeHeaderSemantics: excludeHeaderSemantics,
        titleSpacing: titleSpacing,
        shape: shape,
        toolbarHeight: toolbarHeight,
        leadingWidth: leadingWidth,
        toolbarTextStyle: toolbarTextStyle,
        titleTextStyle: titleTextStyle,
        systemOverlayStyle: systemOverlayStyle,
      );
    } else {
      final bottomHeight = bottom?.preferredSize.height ?? 0;
      final height =
          max(context.mediaQuery.size.height / 5, 120.0) + bottomHeight;
      final padding = titlePadding.resolve(TextDirection.ltr);
      final scaffold = Scaffold.maybeOf(context);
      final parentRoute = ModalRoute.of(context);
      final hasDrawer = scaffold?.hasDrawer ?? false;
      final hasEndDrawer = scaffold?.hasEndDrawer ?? false;
      final canPop = parentRoute?.canPop ?? false;
      final hasLeading = (automaticallyImplyLeading &&
              (hasDrawer || (!hasEndDrawer && canPop))) ||
          leading != null;
      return SliverAppBar(
        key: key,
        leading: leading,
        automaticallyImplyLeading: automaticallyImplyLeading,
        title: null,
        actions: actions,
        flexibleSpace: flexibleSpace ??
            FlexibleSpaceBar(
              title: SingleChildScrollView(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  mainAxisAlignment: MainAxisAlignment.end,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    if (title != null)
                      DefaultTextStyle(
                        style: (Theme.of(context).textTheme.titleLarge ??
                                    Theme.of(context).textTheme.titleLarge ??
                                    Theme.of(context)
                                        .primaryTextTheme
                                        .titleLarge)
                                ?.copyWith(
                              color: borderColor == null ||
                                      backgroundColor != null ||
                                      background != null
                                  ? null
                                  : Theme.of(context).colorScheme.onBackground,
                            ) ??
                            TextStyle(
                              color: borderColor == null ||
                                      backgroundColor != null ||
                                      background != null
                                  ? null
                                  : Theme.of(context).colorScheme.onBackground,
                            ),
                        child: title!,
                      ),
                    if (subtitle != null)
                      DefaultTextStyle(
                        style: Theme.of(context).textTheme.labelSmall?.copyWith(
                                  color: Theme.of(context).disabledColor,
                                ) ??
                            TextStyle(
                              color: Theme.of(context).disabledColor,
                            ),
                        child: subtitle!,
                      ),
                  ],
                ),
              ),
              background: background,
              titlePadding: EdgeInsets.fromLTRB(
                padding.left + (hasLeading ? (leadingWidth ?? 36) : 0),
                padding.top,
                padding.right,
                padding.bottom + (subtitle != null ? 8 : 16) + bottomHeight,
              ),
              centerTitle: false,
            ),
        bottom: bottom,
        elevation: elevation ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? 1
                : 0),
        shadowColor: borderColor ?? Theme.of(context).scaffoldBackgroundColor,
        forceElevated: true,
        backgroundColor:
            backgroundColor ?? Theme.of(context).scaffoldBackgroundColor,
        foregroundColor: foregroundColor ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : Theme.of(context).dividerColor),
        iconTheme: iconTheme ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : IconThemeData(color: Theme.of(context).dividerColor)),
        actionsIconTheme: actionsIconTheme ??
            (borderColor != null ||
                    backgroundColor != null ||
                    background != null
                ? null
                : IconThemeData(color: Theme.of(context).dividerColor)),
        primary: primary,
        centerTitle: true,
        excludeHeaderSemantics: excludeHeaderSemantics,
        titleSpacing: titleSpacing,
        collapsedHeight: _collapsedHeightFromStyle,
        expandedHeight: expandedHeight ?? height,
        floating: _floatingFromStyle,
        pinned: _pinnedFromStyle,
        snap: snap,
        stretch: stretch,
        stretchTriggerOffset: stretchTriggerOffset,
        onStretchTrigger: onStretchTrigger,
        shape: shape,
        toolbarHeight: toolbarHeight,
        leadingWidth: leadingWidth,
        toolbarTextStyle: toolbarTextStyle,
        titleTextStyle: titleTextStyle,
        systemOverlayStyle: systemOverlayStyle,
      );
    }
  }

  bool get _floatingFromStyle {
    switch (scrollStyle) {
      case UIAppBarScrollStyle.floating:
        return true;
      case UIAppBarScrollStyle.hidden:
      case UIAppBarScrollStyle.pinned:
        return false;
      default:
        return floating;
    }
  }

  bool get _pinnedFromStyle {
    switch (scrollStyle) {
      case UIAppBarScrollStyle.pinned:
        return true;
      case UIAppBarScrollStyle.hidden:
      case UIAppBarScrollStyle.floating:
        return false;
      default:
        return pinned;
    }
  }

  double? get _collapsedHeightFromStyle {
    switch (scrollStyle) {
      case UIAppBarScrollStyle.pinned:
        return kToolbarHeight + 1;
      case UIAppBarScrollStyle.hidden:
      case UIAppBarScrollStyle.floating:
        return null;
      default:
        return collapsedHeight;
    }
  }

  bool _useSliverAppBar(BuildContext context) {
    return sliverLayoutWhenModernDesign;
  }
}

class UIAppBarBackground extends StatelessWidget {
  const UIAppBarBackground(
    this.path, {
    super.key,
    this.filterColor = Colors.black87,
    this.fit = BoxFit.cover,
  });

  final String path;
  final BoxFit fit;
  final Color filterColor;

  @override
  Widget build(BuildContext context) {
    return Image(
      image: Asset.image(path),
      fit: fit,
      color: filterColor,
      colorBlendMode: BlendMode.multiply,
    );
  }
}

enum UIAppBarScrollStyle {
  none,
  hidden,
  floating,
  pinned,
}
